#pragma hdrstop

#include <iostream>
#include <fstream>

using namespace std;

int main(int argc, char* argv[])
{
	cout << "* Duels of the Planeswalkers 2013 profile hash generator *" << endl;
	if (argc < 2) // no file specified
	{
		cout << "Usage: D13Hash input_file [output_file]" << endl;
		return 0;
	}

	// set hexadecimal integer output
	cout.flags(ios::hex);

	// read file
	cout << "Opening file " << argv[1] << "...";
	ifstream finput(argv[1], ios_base::in | ios_base::binary | ios_base::ate);
	if (finput.fail())
	{
		cout << endl << "ERROR: File opening failed!" << endl;
		return 1;
	}
	else
		cout << " OK" << endl;
	size_t len = finput.tellg();
	cout << "Length: 0x" << len << endl;
	if (len != 0x10c4)
		cout << "WARNING: non-standard file length (expected 0x10c4)." << endl;
	unsigned char *bytes = new unsigned char[len];
	finput.seekg(0, ios::beg);
	finput.read(bytes, len);
	finput.close();

	// compute hash
	unsigned int hash = 0x811c9dc5;
	for (size_t i=0; i < len; i++)
	{
		hash *= 0x1000193;
		hash ^= bytes[i];
	}

	// free memory
	delete bytes;

	// save result in correct order
	unsigned char *result = new unsigned char[4];
	result[0] = (hash & 0xff);
	result[1] = ((hash & 0xff00) >> 8);
	result[2] = ((hash & 0xff0000) >> 16);
	result[3] = ((hash & 0xff000000) >> 24);

	// write hash to screen
	cout << "Hash: 0x";
	for (int i=0; i<4; i++)
	{
		unsigned int hashpart = (unsigned int)result[i];
		if (hashpart < 16)
			cout << "0";
		cout << hashpart;
	}
	cout << endl;

	if (argc > 2) // output file specified: write hash to file
	{
		cout << "Writing hash to file " << argv[2] << "...";
		ofstream foutput(argv[2], ios_base::out | ios_base::binary);
		if (foutput.fail())
			cout << endl << "ERROR: File writing failed!" << endl;
		else
		{
			foutput.write(result, 4);
			foutput.close();
			cout << " OK" << endl;
        }
	}

	// free memory
	delete result;

	return 0;
}
